"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAppConfig = exports.getAbapDeployConfig = void 0;
const fiori_elements_writer_1 = require("@sap-ux/fiori-elements-writer");
const odata_service_inquirer_1 = require("@sap-ux/odata-service-inquirer");
const i18n_1 = require("../utils/i18n");
const file_helpers_1 = require("../utils/file-helpers");
const constants_1 = require("../utils/constants");
const prompt_state_1 = require("../prompts/prompt-state");
const logger_1 = __importDefault(require("../utils/logger"));
const project_access_1 = require("@sap-ux/project-access");
const path_1 = require("path");
const ui5_info_1 = require("@sap-ux/ui5-info");
/**
 * Generates the deployment configuration for an ABAP application.
 *
 * @param {AppInfo} app - Application info containing `url` and `repoName`.
 * @param {QfaJsonConfig} qfaJson - The QFA JSON configuration containing app details.
 * @returns {AbapDeployConfig} The deployment configuration containing `target` and `app` info.
 */
const getAbapDeployConfig = (app, qfaJson) => {
    return {
        target: {
            url: prompt_state_1.PromptState.baseURL,
            client: prompt_state_1.PromptState.sapClient,
            destination: app.repoName
        },
        app: {
            name: qfaJson.deploymentDetails.repositoryName,
            package: qfaJson.metadata.package,
            description: qfaJson.deploymentDetails.repositoryDescription,
            transport: 'REPLACE_WITH_TRANSPORT'
        }
    };
};
exports.getAbapDeployConfig = getAbapDeployConfig;
/**
 * Fetches the metadata of a given service from the provided ABAP service provider.
 *
 * @param {AbapServiceProvider} provider - The ABAP service provider instance.
 * @param {string} serviceUrl - The URL of the service to retrieve metadata for.
 * @returns {Promise<any>} - A promise resolving to the service metadata.
 */
const fetchServiceMetadata = async (provider, serviceUrl) => {
    try {
        return await provider.service(serviceUrl).metadata();
    }
    catch (err) {
        logger_1.default.logger?.error((0, i18n_1.t)('error.metadataFetchError', { error: err.message }));
    }
};
/**
 * Gets the application configuration based on the provided user answers and manifest data.
 * This configuration will be used to initialize a new Fiori application.
 *
 * @param {AppInfo} app - Selected app information.
 * @param {string} extractedProjectPath - Path where the app files are extracted.
 * @param {QfaJsonConfig} qfaJson - The QFA JSON configuration containing app details.
 * @param {Editor} fs - The file system editor to manipulate project files.
 * @returns {Promise<FioriElementsApp<LROPSettings>>} - A promise resolving to the generated app configuration.
 * @throws {Error} - Throws an error if there are issues generating the configuration.
 */
async function getAppConfig(app, extractedProjectPath, qfaJson, fs) {
    try {
        const manifest = (0, file_helpers_1.readManifest)((0, path_1.join)(extractedProjectPath, project_access_1.FileName.Manifest), fs);
        const serviceProvider = prompt_state_1.PromptState.systemSelection?.connectedSystem?.serviceProvider;
        if (!manifest?.['sap.app']?.dataSources) {
            logger_1.default.logger?.error((0, i18n_1.t)('error.dataSourcesNotFound'));
        }
        const odataVersion = manifest?.['sap.app']?.dataSources?.mainService?.settings?.odataVersion === '4.0'
            ? odata_service_inquirer_1.OdataVersion.v4
            : odata_service_inquirer_1.OdataVersion.v2;
        // Fetch metadata for the service
        const metadata = await fetchServiceMetadata(serviceProvider, manifest?.['sap.app']?.dataSources?.mainService.uri ?? '');
        // Fetch latest UI5 versions from npm
        const ui5Versions = await (0, ui5_info_1.getUI5Versions)({ onlyNpmVersion: true });
        const localVersion = ui5Versions[0]?.version;
        const appConfig = {
            app: {
                id: app.appId,
                title: app.title,
                description: app.description,
                sourceTemplate: {
                    id: constants_1.fioriAppSourcetemplateId
                },
                projectType: 'EDMXBackend',
                flpAppId: `${app.appId.replace(/[-_.#]/g, '')}-tile`
            },
            package: {
                name: app.appId,
                description: app.description,
                devDependencies: {},
                scripts: {},
                version: manifest?.['sap.app']?.applicationVersion?.version ?? '0.0.1'
            },
            template: {
                type: fiori_elements_writer_1.TemplateType.ListReportObjectPage,
                settings: {
                    entityConfig: {
                        mainEntityName: qfaJson.serviceBindingDetails.mainEntityName
                    }
                }
            },
            service: {
                path: manifest?.['sap.app']?.dataSources?.mainService.uri,
                version: odataVersion,
                metadata: metadata,
                url: serviceProvider.defaults.baseURL
            },
            appOptions: {
                addAnnotations: odataVersion === odata_service_inquirer_1.OdataVersion.v4,
                addTests: true
            },
            ui5: {
                localVersion
            }
        };
        return appConfig;
    }
    catch (error) {
        logger_1.default.logger?.error((0, i18n_1.t)('error.appConfigGenError', { error: error.message }));
        throw error;
    }
}
exports.getAppConfig = getAppConfig;
//# sourceMappingURL=app-config.js.map