"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PromptNames = void 0;
const yeoman_generator_1 = __importDefault(require("yeoman-generator"));
const logger_1 = __importDefault(require("../utils/logger"));
const yeoman_ui_types_1 = require("@sap-devx/yeoman-ui-types");
const feature_toggle_1 = require("@sap-ux/feature-toggle");
const constants_1 = require("../utils/constants");
const i18n_1 = require("../utils/i18n");
const download_utils_1 = require("../utils/download-utils");
const telemetryEvents_1 = require("../telemetryEvents");
const fiori_generator_shared_1 = require("@sap-ux/fiori-generator-shared");
const prompts_1 = require("../prompts/prompts");
const fiori_elements_writer_1 = require("@sap-ux/fiori-elements-writer");
const path_1 = require("path");
const os_1 = require("os");
const event_hook_1 = require("../utils/event-hook");
const ui5_info_1 = require("@sap-ux/ui5-info");
const launch_config_1 = require("@sap-ux/launch-config");
const btp_utils_1 = require("@sap-ux/btp-utils");
const odata_service_inquirer_1 = require("@sap-ux/odata-service-inquirer");
const fiori_tools_settings_1 = require("@sap-ux/fiori-tools-settings");
const abap_deploy_config_writer_1 = require("@sap-ux/abap-deploy-config-writer");
const prompt_state_1 = require("../prompts/prompt-state");
const types_1 = require("./types");
Object.defineProperty(exports, "PromptNames", { enumerable: true, get: function () { return types_1.PromptNames; } });
const app_config_1 = require("./app-config");
const file_helpers_1 = require("../utils/file-helpers");
const updates_1 = require("../utils/updates");
const prompt_helpers_1 = require("../prompts/prompt-helpers");
const validators_1 = require("../utils/validators");
const project_access_1 = require("@sap-ux/project-access");
/**
 * Generator class for downloading a basic app from a repository.
 * This class handles the process of app selection, downloading the app and generating a fiori app from the downloaded app
 */
class default_1 extends yeoman_generator_1.default {
    appWizard;
    vscode;
    appRootPath;
    prompts;
    answers = constants_1.defaultAnswers;
    options;
    projectPath;
    extractedProjectPath;
    setPromptsCallback;
    /**
     * Constructor for Downloading App.
     *
     * @param args - arguments passed to the generator
     * @param opts - options passed to the generator
     */
    constructor(args, opts) {
        super(args, opts);
        // Initialise properties from options
        this.appWizard = opts.appWizard ?? yeoman_ui_types_1.AppWizard.create(opts);
        this.vscode = opts.vscode;
        this.appRootPath = opts?.appRootPath ?? (0, fiori_generator_shared_1.getDefaultTargetFolder)(this.vscode) ?? this.destinationRoot();
        this.options = opts;
        // Configure logging
        logger_1.default.configureLogging(this.rootGeneratorName(), this.log, this.options.logWrapper, this.options.logLevel, this.options.logger, this.vscode);
        // Initialise prompts and callbacks if not launched as a subgenerator
        this.appWizard.setHeaderTitle(constants_1.generatorTitle);
        this.prompts = new yeoman_ui_types_1.Prompts((0, prompt_helpers_1.getYUIDetails)());
        this.setPromptsCallback = (fn) => {
            if (this.prompts) {
                this.prompts.setCallback(fn);
            }
        };
    }
    /**
     * Initialises necessary settings and telemetry for the generator.
     */
    async initializing() {
        if (this.env.conflicter) {
            this.env.conflicter.force = this.options.force ?? true;
        }
        // Initialise telemetry settings
        await fiori_generator_shared_1.TelemetryHelper.initTelemetrySettings({
            consumerModule: {
                name: constants_1.generatorName,
                version: this.rootGeneratorVersion()
            },
            internalFeature: (0, feature_toggle_1.isInternalFeaturesSettingEnabled)(),
            watchTelemetrySettingStore: false
        });
    }
    /**
     * Prompts the user for application details and downloads the app.
     */
    async prompting() {
        const quickDeployedAppConfig = this.options?.data?.quickDeployedAppConfig;
        const questions = await (0, prompts_1.getPrompts)(this.appRootPath, quickDeployedAppConfig);
        const answers = await this.prompt(questions);
        const { targetFolder } = answers;
        if (quickDeployedAppConfig?.appId) {
            // Handle quick deployed app download where prompts for system selection and app selection are not displayed
            // Only target folder prompt is shown
            this.answers.targetFolder = targetFolder;
            this.answers.systemSelection = prompt_state_1.PromptState.systemSelection;
            this.answers.selectedApp = answers.selectedApp;
        }
        else {
            // Handle app download where prompts for system selection and app selection are shown
            Object.assign(this.answers, answers);
        }
        if ((0, validators_1.isValidPromptState)(this.answers.targetFolder, this.answers.selectedApp.appId)) {
            this.projectPath = (0, path_1.join)(this.answers.targetFolder, this.answers.selectedApp.appId);
            this.extractedProjectPath = (0, path_1.join)(this.projectPath, constants_1.extractedFilePath);
        }
    }
    /**
     * Writes the configuration files for the project, including deployment config, and README.
     */
    async writing() {
        // Extract downloaded app
        const archive = prompt_state_1.PromptState.downloadedAppPackage;
        await (0, download_utils_1.extractZip)(this.extractedProjectPath, archive, this.fs);
        // Check if the qfa.json file
        const qfaJsonFilePath = (0, path_1.join)(this.extractedProjectPath, constants_1.qfaJsonFileName);
        if (this.fs.exists(qfaJsonFilePath)) {
            const qfaJson = (0, file_helpers_1.makeValidJson)(qfaJsonFilePath, this.fs);
            // Generate project files
            (0, validators_1.validateQfaJsonFile)(qfaJson);
            // Generate app config
            const config = await (0, app_config_1.getAppConfig)(this.answers.selectedApp, this.extractedProjectPath, qfaJson, this.fs);
            await (0, fiori_elements_writer_1.generate)(this.projectPath, config, this.fs);
            // Generate deploy config
            const deployConfig = (0, app_config_1.getAbapDeployConfig)(this.answers.selectedApp, qfaJson);
            await (0, abap_deploy_config_writer_1.generate)(this.projectPath, deployConfig, undefined, this.fs);
            if (this.vscode) {
                // Generate Fiori launch config
                const fioriOptions = this._getLaunchConfig(config);
                // Create launch configuration
                await (0, launch_config_1.createLaunchConfig)(this.projectPath, fioriOptions, this.fs, logger_1.default.logger);
                (0, fiori_tools_settings_1.writeApplicationInfoSettings)(this.projectPath);
            }
            // Generate README
            const readMeConfig = this._getReadMeConfig(config);
            (0, fiori_generator_shared_1.generateReadMe)(this.projectPath, readMeConfig, this.fs);
            // Replace webapp files with downloaded app files
            await (0, updates_1.replaceWebappFiles)(this.projectPath, this.extractedProjectPath, this.fs);
            await (0, updates_1.validateAndUpdateManifestUI5Version)((0, path_1.join)(this.projectPath, project_access_1.DirName.Webapp, project_access_1.FileName.Manifest), this.fs);
            // Clean up extracted project files
            this.fs.delete(this.extractedProjectPath);
        }
        else {
            logger_1.default.logger?.error((0, i18n_1.t)('error.qfaJsonNotFound', { jsonFileName: constants_1.qfaJsonFileName }));
        }
    }
    /**
     * Returns the configuration for the README file.
     *
     * @param config - The app configuration object.
     * @returns {ReadMe} The configuration for generating the README.
     */
    _getReadMeConfig(config) {
        const readMeConfig = {
            appName: config.app.id,
            appTitle: config.app.title ?? '',
            appNamespace: config.app.id.substring(0, config.app.id.lastIndexOf('.')),
            appDescription: (0, i18n_1.t)('readMe.appDescription'),
            ui5Theme: (0, ui5_info_1.getDefaultUI5Theme)(config.ui5?.version),
            generatorName: constants_1.generatorName,
            generatorVersion: this.rootGeneratorVersion(),
            ui5Version: config.ui5?.version ?? '',
            template: fiori_elements_writer_1.TemplateType.ListReportObjectPage,
            serviceUrl: config.service.url,
            launchText: (0, i18n_1.t)('readMe.launchText')
        };
        return readMeConfig;
    }
    /**
     * Returns the configuration for launching the app with Fiori options.
     *
     * @param config - The app configuration object.
     * @returns {FioriOptions} The launch configuration options.
     */
    _getLaunchConfig(config) {
        const debugOptions = {
            vscode: this.vscode,
            addStartCmd: true,
            sapClientParam: prompt_state_1.PromptState.sapClient,
            flpAppId: config.app.flpAppId ?? config.app.id,
            flpSandboxAvailable: true,
            isAppStudio: (0, btp_utils_1.isAppStudio)(),
            odataVersion: config.service.version === odata_service_inquirer_1.OdataVersion.v2 ? '2.0' : '4.0'
        };
        const fioriOptions = {
            name: config.app.id,
            projectRoot: this.projectPath,
            /**
             * The `enableVSCodeReload` property is set to `false` to prevent automatic reloading of the VS Code workspace
             * after the app generation process. This is necessary to ensure that the `.vscode/launch-config.json` file is
             * written to disk before the workspace reload occurs. See {@link _handlePostAppGeneration} for details.
             */
            enableVSCodeReload: false,
            debugOptions
        };
        return fioriOptions;
    }
    /**
     * Installs npm dependencies for the project.
     */
    async install() {
        if (!this.options.skipInstall) {
            try {
                await this._runNpmInstall(this.projectPath);
            }
            catch (error) {
                logger_1.default.logger?.error((0, i18n_1.t)('error.installationErrors.npmInstall', { error }));
            }
        }
        else {
            logger_1.default.logger?.info((0, i18n_1.t)('info.installationErrors.skippedInstallation'));
        }
    }
    /**
     * Runs npm install in the specified path.
     *
     * @param path - The path to run npm install.
     */
    async _runNpmInstall(path) {
        const npm = (0, os_1.platform)() === 'win32' ? 'npm.cmd' : 'npm';
        // install dependencies
        await this.spawnCommand(npm, ['install', '--no-audit', '--no-fund', '--silent', '--prefer-offline', '--no-progress'], {
            cwd: path
        });
    }
    /**
     * Responsible for updating workspace folders and running post-generation commands if defined.
     */
    async _handlePostAppGeneration() {
        /**
         * `enableVSCodeReload` is set to false when generating launch config here {@link _getLaunchConfig}.
         * This prevents issues where the `.vscode/launch-config.json` file may not be written to disk due to the timing of mem-fs commits.
         *
         * In Yeoman, a commit occurs between the writing phase and the end phase. If no workspace is open in VS Code and the generated
         * app is added to the workspace, VS Code automatically reloads the window. However, by this point in the end phase, the in-memory file system
         * (mem-fs) has written all files except for `.vscode/launch-config.json`, because the commit happens before the end phase
         * causing it to be missed when the workspace reload occurs.
         *
         * Workflow:
         * 1. **Workspace URI**: The `updateWorkspaceFolders` object is created with the project folder's path, the project name,
         *    and the VS Code instance to handle workspace folder updates.
         * 2. **Update Workspace Folders**: The `updateWorkspaceFoldersIfNeeded` function is called to update the workspace folders,
         *    if necessary. See {@link updateWorkspaceFoldersIfNeeded} for details.
         * 3. **Run Post-Generation Commands**: If defined, post-generation commands from `options.data?.postGenCommands` are executed
         *    using the `runPostAppGenHook` function. This allows for additional setup or configuration tasks to be performed after
         *    the app generation process.
         */
        if (this.vscode) {
            const updateWorkspaceFolders = {
                uri: this.vscode?.Uri?.file((0, path_1.join)(this.projectPath)),
                projectName: (0, path_1.basename)(this.projectPath),
                vscode: this.vscode
            };
            (0, launch_config_1.updateWorkspaceFoldersIfNeeded)(updateWorkspaceFolders);
        }
        if (this.options.data?.postGenCommand) {
            await (0, event_hook_1.runPostAppGenHook)({
                path: this.projectPath,
                vscodeInstance: this.vscode,
                postGenCommand: this.options.data?.postGenCommand
            });
        }
    }
    /**
     * Finalises the generator process by creating launch configurations and running post-generation hooks.
     */
    async end() {
        try {
            this.appWizard.showInformation((0, i18n_1.t)('info.repoAppDownloadCompleteMsg'), yeoman_ui_types_1.MessageType.notification);
            await (0, fiori_generator_shared_1.sendTelemetry)(telemetryEvents_1.EventName.GENERATION_SUCCESS, fiori_generator_shared_1.TelemetryHelper.createTelemetryData({
                appType: 'repo-app-import-sub-generator',
                ...this.options.telemetryData
            }) ?? {}).catch((error) => {
                logger_1.default.logger?.error((0, i18n_1.t)('error.telemetry', { error: error.message }));
            });
            await this._handlePostAppGeneration();
        }
        catch (error) {
            logger_1.default.logger?.error((0, i18n_1.t)('error.endPhase', { error: error.message }));
        }
    }
}
exports.default = default_1;
//# sourceMappingURL=index.js.map