"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.replaceWebappFiles = exports.validateAndUpdateManifestUI5Version = void 0;
const path_1 = require("path");
const project_access_1 = require("@sap-ux/project-access");
const i18n_1 = require("./i18n");
const logger_1 = __importDefault(require("./logger"));
const feature_toggle_1 = require("@sap-ux/feature-toggle");
const ui5_info_1 = require("@sap-ux/ui5-info");
const file_helpers_1 = require("./file-helpers");
const constants_1 = require("./constants");
/**
 * Validates and updates the UI5 version in the manifest.
 *
 * - If the minUI5Version in the manifest is not found in the list of released UI5 versions,
 *   it updates the manifest with the closest released version.
 * - If internal features are enabled, it sets the minUI5Version to '${sap.ui5.dist.version}'.
 *
 * @param {string} manifestFilePath - The manifest file path.
 * @param {Editor} fs - The file system editor instance.
 * @returns {Promise<Manifest>} - The updated manifest object.
 * @throws {Error} - Throws an error if the manifest structure is invalid or no fallback version is available.
 */
async function validateAndUpdateManifestUI5Version(manifestFilePath, fs) {
    const manifestJson = (0, file_helpers_1.readManifest)(manifestFilePath, fs);
    if (!manifestJson?.['sap.ui5']?.dependencies || !manifestJson?.['sap.app']?.sourceTemplate) {
        // Check if the manifest structure is valid) {
        throw new Error((0, i18n_1.t)('error.readManifestErrors.invalidManifestStructureError'));
    }
    const manifestUi5Version = manifestJson['sap.ui5']?.dependencies?.minUI5Version;
    const availableUI5Versions = await (0, ui5_info_1.getUI5Versions)({ includeMaintained: true });
    // Check if the manifest version exists in the list of released versions
    const ui5VersionAvailable = availableUI5Versions.find((ui5Version) => ui5Version.version === manifestUi5Version);
    if (ui5VersionAvailable) {
        // Return the manifest as it is if the version is valid
        // No changes needed
    }
    else if ((0, feature_toggle_1.isInternalFeaturesSettingEnabled)()) {
        // Handle internal features setting
        manifestJson['sap.ui5'].dependencies.minUI5Version = '${sap.ui5.dist.version}';
    }
    else {
        // Handle fallback to the closest released version
        const closestAvailableUi5Version = availableUI5Versions[0]?.version;
        manifestJson['sap.ui5'].dependencies.minUI5Version = closestAvailableUi5Version;
    }
    manifestJson['sap.app'].sourceTemplate.id = constants_1.fioriAppSourcetemplateId;
    // update manifest at extracted path
    fs.writeJSON(manifestFilePath, manifestJson, undefined, 2);
}
exports.validateAndUpdateManifestUI5Version = validateAndUpdateManifestUI5Version;
/**
 * Replaces the specified files in the `webapp` directory with the corresponding files from the `extractedPath`.
 *
 * @param {string} projectPath - The path to the downloaded App.
 * @param {string} extractedPath - The path from which files will be copied.
 * @param {Editor} fs - The file system editor instance to modify files in memory.
 */
async function replaceWebappFiles(projectPath, extractedPath, fs) {
    try {
        const webappPath = (0, path_1.join)(projectPath, project_access_1.DirName.Webapp);
        // Define the paths of the files to be replaced
        const filesToReplace = [
            { webappFile: project_access_1.FileName.Manifest, extractedFile: project_access_1.FileName.Manifest },
            { webappFile: (0, path_1.join)('i18n', 'i18n.properties'), extractedFile: (0, path_1.join)('i18n', 'i18n.properties') },
            { webappFile: 'index.html', extractedFile: 'index.html' },
            { webappFile: 'Component.js', extractedFile: 'component.js' }
        ];
        // Loop through each file and perform the replacement
        for (const { webappFile, extractedFile } of filesToReplace) {
            const webappFilePath = (0, path_1.join)(webappPath, webappFile);
            const extractedFilePath = (0, path_1.join)(extractedPath, extractedFile);
            // Check if the extracted file exists before replacing
            if (fs.exists(extractedFilePath)) {
                fs.copy(extractedFilePath, webappFilePath);
            }
            else {
                logger_1.default.logger?.warn((0, i18n_1.t)('warn.extractedFileNotFound', { extractedFilePath }));
            }
        }
    }
    catch (error) {
        logger_1.default.logger?.error((0, i18n_1.t)('error.replaceWebappFilesError', { error }));
    }
}
exports.replaceWebappFiles = replaceWebappFiles;
//# sourceMappingURL=updates.js.map