"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateAppSelection = exports.isValidPromptState = exports.validateQfaJsonFile = void 0;
const i18n_1 = require("../utils/i18n");
const logger_1 = __importDefault(require("../utils/logger"));
const prompt_state_1 = require("../prompts/prompt-state");
const guided_answers_helper_1 = require("@sap-ux/guided-answers-helper");
const inquirer_common_1 = require("@sap-ux/inquirer-common");
const download_utils_1 = require("../utils/download-utils");
/**
 * Validates the metadata section of the app configuration.
 *
 * @param {QfaJsonConfig['metadata']} metadata - The metadata object.
 * @returns {boolean} - Returns true if valid, false otherwise.
 */
const validateMetadata = (metadata) => {
    if (!metadata.package || typeof metadata.package !== 'string') {
        logger_1.default.logger?.error((0, i18n_1.t)('error.invalidMetadataPackage'));
        return false;
    }
    return true;
};
/**
 * Validates the service binding details section of the app configuration.
 *
 * @param {QfaJsonConfig['serviceBindingDetails']} serviceBinding - The service binding details object.
 * @returns {boolean} - Returns true if valid, false otherwise.
 */
const validateServiceBindingDetails = (serviceBinding) => {
    if (!serviceBinding.serviceName || typeof serviceBinding.serviceName !== 'string') {
        logger_1.default.logger?.error((0, i18n_1.t)('error.invalidServiceName'));
        return false;
    }
    if (!serviceBinding.serviceVersion || typeof serviceBinding.serviceVersion !== 'string') {
        logger_1.default.logger?.error((0, i18n_1.t)('error.invalidServiceVersion'));
        return false;
    }
    if (!serviceBinding.mainEntityName || typeof serviceBinding.mainEntityName !== 'string') {
        logger_1.default.logger?.error((0, i18n_1.t)('error.invalidMainEntityName'));
        return false;
    }
    return true;
};
/**
 * Validates the project attribute section of the app configuration.
 *
 * @param {QfaJsonConfig['projectAttribute']} projectAttribute - The project attribute object.
 * @returns {boolean} - Returns true if valid, false otherwise.
 */
const validateProjectAttribute = (projectAttribute) => {
    if (!projectAttribute.moduleName || typeof projectAttribute.moduleName !== 'string') {
        logger_1.default.logger?.error((0, i18n_1.t)('error.invalidModuleName'));
        return false;
    }
    return true;
};
/**
 * Validates the deployment details section of the app configuration.
 *
 * @param {QfaJsonConfig['deploymentDetails']} deploymentDetails - The deployment details object.
 * @returns {boolean} - Returns true if valid, false otherwise.
 */
const validateDeploymentDetails = (deploymentDetails) => {
    if (!deploymentDetails.repositoryName) {
        logger_1.default.logger?.error((0, i18n_1.t)('error.invalidRepositoryName'));
        return false;
    }
    return true;
};
/**
 * Validates the entire app configuration.
 *
 * @param {QfaJsonConfig} config - The QFA JSON configuration containing app details.
 * @returns {boolean} - Returns true if the configuration is valid, false otherwise.
 */
const validateQfaJsonFile = (config) => {
    return (validateMetadata(config.metadata) &&
        validateServiceBindingDetails(config.serviceBindingDetails) &&
        validateProjectAttribute(config.projectAttribute) &&
        validateDeploymentDetails(config.deploymentDetails));
};
exports.validateQfaJsonFile = validateQfaJsonFile;
/**
 * Validates the prompt state for the app download process.
 *
 * @param {string} targetFolder - The target folder for the app download.
 * @param {string} [appId] - The selected app id.
 * @returns {boolean} - Returns true if the prompt state is valid, false otherwise.
 */
const isValidPromptState = (targetFolder, appId) => {
    return !!(prompt_state_1.PromptState.systemSelection.connectedSystem?.serviceProvider && appId && targetFolder);
};
exports.isValidPromptState = isValidPromptState;
/**
 * Generates a help link for the "App Not Found" error.
 *
 * @returns {Promise<ValidationLink>} - A promise resolving to a validation link for the error.
 */
async function generateAppNotFoundHelpLink() {
    return inquirer_common_1.ErrorHandler.getHelpLink(guided_answers_helper_1.HELP_NODES.ADT_APP_NOT_FOUND_ERROR, inquirer_common_1.ERROR_TYPE.INTERNAL_SERVER_ERROR, (0, i18n_1.t)('error.noAppsDeployed'));
}
/**
 * Validates the app selection and handles app download if applicable.
 *
 * @param answers - The selected app information.
 * @param appList - The list of available apps.
 * @param quickDeployedAppConfig - The quick deployed app configuration.
 * @returns A promise resolving to a boolean or a validation error message.
 */
async function validateAppSelection(answers, appList, quickDeployedAppConfig) {
    // Quick deploy config exists but no apps found
    if (quickDeployedAppConfig?.appId && appList.length === 0) {
        return await generateAppNotFoundHelpLink();
    }
    // No apps available at all
    if (appList.length === 0) {
        return await generateAppNotFoundHelpLink();
    }
    // Valid app selected, try to download
    if (answers?.appId) {
        try {
            await (0, download_utils_1.downloadApp)(answers.repoName);
            return true;
        }
        catch (error) {
            return (0, i18n_1.t)('error.appDownloadErrors.appDownloadFailure', { error: error.message });
        }
    }
    return false;
}
exports.validateAppSelection = validateAppSelection;
//# sourceMappingURL=validators.js.map